#!/usr/bin/env python

"""
Handlers for a person for whom scheduling is performed.
"""

from email.mime.text import MIMEText
from imiptools.config import MANAGER_PATH, MANAGER_URL
from imiptools.content import Handler, to_part
from imiptools.handlers.common import CommonFreebusy
from socket import gethostname
from vCalendar import to_node

def get_manager_url():
    url_base = MANAGER_URL or "http://%s/" % gethostname()
    return "%s/%s" % (url_base.rstrip("/"), MANAGER_PATH.lstrip("/"))

class PersonHandler(Handler):

    "Handling mechanisms specific to people."

    def _record_and_deliver(self, objtype, from_organiser=True, queue=False):

        oa = self.require_organiser_and_attendees(from_organiser)
        if not oa:
            return False

        (organiser, organiser_attr), attendees = organiser_item, attendees = oa

        # Validate the organiser or attendee, ignoring spoofed requests.

        if not self.validate_identities(from_organiser and [organiser_item] or attendees.items()):
            return False

        # Process each attendee separately.

        if from_organiser:
            for attendee, attendee_attr in attendees.items():

                if not self.have_new_object(attendee, objtype):
                    continue

                # Store the object and queue any request.

                self.store.set_event(attendee, self.uid, to_node(
                    {objtype : [(self.details, {})]}
                    ))

                if queue:
                    self.store.queue_request(attendee, self.uid)

        # As organiser, update attendance.

        else:
            obj = self.get_object(organiser, objtype)

            if obj and self.have_new_object(organiser, objtype, obj):
                attendee_map = self.get_value_map("ATTENDEE")

                for attendee, attendee_attr in attendees.items():

                    # Update attendance in the loaded object.

                    attendee_map[attendee] = attendee_attr

                # Set the new details and store the object.

                obj["ATTENDEE"] = attendee_map.items()

                self.store.set_event(organiser, self.uid, to_node(
                    {objtype : [(obj, {})]}
                    ))

        return True

    def reply(self):

        "Wrap any valid message and pass it on to the recipient."

        attendee = self.get_value("ATTENDEE")
        if attendee:
            return "REPLY", MIMEText("A reply has been received from %s." % attendee)

class Event(PersonHandler):

    "An event handler."

    def add(self):

        # NOTE: Queue a suggested modification to any active event.

        # The message is now wrapped and passed on to the recipient.

        return "ADD", MIMEText("An addition to an event has been received.")

    def cancel(self):

        # NOTE: Queue a suggested modification to any active event.

        # The message is now wrapped and passed on to the recipient.

        return "CANCEL", MIMEText("A cancellation has been received.")

    def counter(self):

        # NOTE: Queue a suggested modification to any active event.

        # The message is now wrapped and passed on to the recipient.

        return "COUNTER", MIMEText("A counter proposal has been received.")

    def declinecounter(self):

        # NOTE: Queue a suggested modification to any active event.

        # The message is now wrapped and passed on to the recipient.

        return "DECLINECOUNTER", MIMEText("A declining counter proposal has been received.")

    def publish(self):

        # NOTE: Register details of any relevant event.

        # The message is now wrapped and passed on to the recipient.

        return "PUBLISH", MIMEText("Details of an event have been received.")

    def refresh(self):

        # NOTE: Update details of any active event.

        # The message is now wrapped and passed on to the recipient.

        return "REFRESH", MIMEText("An event update has been received.")

    def reply(self):

        "Record replies and notify the recipient."

        self._record_and_deliver("VEVENT", from_organiser=False, queue=False)
        return PersonHandler.reply(self)

    def request(self):

        "Hold requests and notify the recipient."

        self._record_and_deliver("VEVENT", from_organiser=True, queue=True)

        # The message is now wrapped and passed on to the recipient.

        url = "%s/%s" % (get_manager_url().rstrip("/"), self.uid)
        return "REQUEST", MIMEText("A request has been queued and can be viewed here: %s" % url)

class Freebusy(PersonHandler, CommonFreebusy):

    "A free/busy handler."

    def publish(self):

        # NOTE: Register free/busy information.

        # The message is now wrapped and passed on to the recipient.

        return "PUBLISH", MIMEText("Details of a contact's availability have been received.")

    def reply(self):

        "Record replies and notify the recipient."

        self._record_and_deliver("VFREEBUSY", from_organiser=False, queue=False)
        return PersonHandler.reply(self)

    def request(self):

        """
        Respond to a request by preparing a reply containing free/busy
        information for each indicated attendee.
        """

        # NOTE: This should be subject to policy/preferences.

        return CommonFreebusy.request(self)

class Journal(PersonHandler):

    "A journal entry handler."

    def add(self):

        # NOTE: Queue a suggested modification to any active entry.

        # The message is now wrapped and passed on to the recipient.

        return "ADD", MIMEText("An addition to a journal entry has been received.")

    def cancel(self):

        # NOTE: Queue a suggested modification to any active entry.

        # The message is now wrapped and passed on to the recipient.

        return "CANCEL", MIMEText("A cancellation has been received.")

    def publish(self):

        # NOTE: Register details of any relevant entry.

        # The message is now wrapped and passed on to the recipient.

        return "PUBLISH", MIMEText("Details of a journal entry have been received.")

class Todo(PersonHandler):

    "A to-do item handler."

    def add(self):

        # NOTE: Queue a suggested modification to any active item.

        # The message is now wrapped and passed on to the recipient.

        return "ADD", MIMEText("An addition to an item has been received.")

    def cancel(self):

        # NOTE: Queue a suggested modification to any active item.

        # The message is now wrapped and passed on to the recipient.

        return "CANCEL", MIMEText("A cancellation has been received.")

    def counter(self):

        # NOTE: Queue a suggested modification to any active item.

        # The message is now wrapped and passed on to the recipient.

        return "COUNTER", MIMEText("A counter proposal has been received.")

    def declinecounter(self):

        # NOTE: Queue a suggested modification to any active item.

        # The message is now wrapped and passed on to the recipient.

        return "DECLINECOUNTER", MIMEText("A declining counter proposal has been received.")

    def publish(self):

        # NOTE: Register details of any relevant item.

        # The message is now wrapped and passed on to the recipient.

        return "PUBLISH", MIMEText("Details of an item have been received.")

    def refresh(self):

        # NOTE: Update details of any active item.

        # The message is now wrapped and passed on to the recipient.

        return "REFRESH", MIMEText("An item update has been received.")

    def reply(self):

        "Record replies and notify the recipient."

        self._record_and_deliver("VTODO", from_organiser=False, queue=False)
        return PersonHandler.reply(self)

    def request(self):

        "Hold requests and notify the recipient."

        self._record_and_deliver("VTODO", from_organiser=True, queue=True)

        # The message is now wrapped and passed on to the recipient.

        return "REQUEST", MIMEText("A request has been queued.")

# Handler registry.

handlers = [
    ("VFREEBUSY",   Freebusy),
    ("VEVENT",      Event),
    ("VTODO",       Todo),
    ("VJOURNAL",    Journal),
    ]

# vim: tabstop=4 expandtab shiftwidth=4
